
!UiTreeView methodsFor: 'model updates' stamp: 'mt 10/14/2010 10:50:48.653'!
changeRowsFor: aParentModelIndex from: start to: end

	| index item |
	
	(self viewport isExpanded: aParentModelIndex)
		ifFalse: [^ self].
	
	start to: end do: [:row |
		1 to: (self model columnCountFor: aParentModelIndex) do: [:col |
			index := self model indexRow: row column: col parent: aParentModelIndex.
			item := (self itemDelegateFor: index) itemFor: index.
			self viewport replaceMorphWith: item atIndex: index.
			item fadeIn]].
	
	self viewport
		updateLayout;
		updateExtent.
		
	self updateScrollBars.! !

!UiTreeView methodsFor: 'model access' stamp: 'mt 10/11/2010 18:05:14.956'!
fillViewportFor: aModelIndex animated: aBoolean

	| index item |
	self model ifNil: [^ self].
	
	1 to: (self model rowCountFor: aModelIndex) do: [:row |
		1 to: (self model columnCountFor: aModelIndex) do: [:col |
			index := self model indexRow: row column: col parent: aModelIndex.
			item := (self itemDelegateFor: index) itemFor: index.
			self viewport addMorph: item atIndex: index.
			aBoolean ifTrue: [item fadeIn]]].! !

!UiTreeView methodsFor: 'model updates' stamp: 'mt 10/14/2010 10:50:31.553'!
insertRowsFor: aParentModelIndex from: start to: end

	| index item |
	
	(self viewport isExpanded: aParentModelIndex)
		ifFalse: [^ self].
	
	start to: end do: [:row |
		1 to: (self model columnCountFor: aParentModelIndex) do: [:col |
			index := self model indexRow: row column: col parent: aParentModelIndex.
			item := (self itemDelegateFor: index) itemFor: index.
			self viewport addMorph: item atIndex: index.
			item fadeIn]].
	
	self viewport
		updateLayout;
		updateExtent.
		
	self updateScrollBars.! !

!UiTreeView methodsFor: 'initialization' stamp: 'mt 10/14/2010 10:21:30.447'!
model: aModel

	self model ifNotNil: [:m | m disconnectAllSignals].
	
	super model: aModel.
	self reset.
	
	self model ifNotNil: [:m |
		self connect: m signal: #modelReset toSelector: #reset.
		self connect: m signal: #rowsInsertedFor:from:to: toSelector: #insertRowsFor:from:to:.
		self connect: m signal: #rowsChangedFor:from:to: toSelector: #changeRowsFor:from:to:].! !

!UiTreeView methodsFor: 'model access' stamp: 'mt 10/10/2010 14:15:33.809'!
reset
	self viewport clear.
	
	self fillViewportFor: nil animated: false.
	self resetHeaderItems.
	
	self viewport
		updateLayout;
		updateExtent.
		
	self updateScrollBars.! !

!UiTreeView methodsFor: 'model access' stamp: 'mt 10/11/2010 18:05:08.738'!
resetHeaderItems

	| index item |
	self model ifNil: [^ self].
	"1 to: (self model rowCountFor: nil) do: [:row |
		index := self model indexRow: row column: 0 parent: nil.
		item := (self itemDelegateFor: index) itemFor: index.
		item width: verticalHeader width.
		self viewport setHeaderItem: item atRow: row]."
	
	1 to: (self model columnCountFor: nil) do: [:col |
		index := self model indexRow: 0 column: col parent: nil.
		item := (self itemDelegateFor: index) itemFor: index.
		horizontalHeader height: (horizontalHeader height max: item height).
		self viewport setHeaderItem: item atColumn: col].! !

!UiTreeView methodsFor: 'model access' stamp: 'mt 10/10/2010 12:04:23.588'!
toggleIndex: aModelIndex

	| row |
	
	row := self viewport rowFor: aModelIndex.
	
	row isExpanded
		ifTrue: [
			row items first setProperty: #isExpanded toValue: false.
			self viewport collapseRow: row]
		ifFalse: [
			row items first setProperty: #isExpanded toValue: true.
			self
				fillViewportFor: row firstModelIndex
				animated: true].
	
	self viewport
		updateLayout;
		updateExtent.
		
	self updateScrollBars.! !

!UiModel methodsFor: 'counts' stamp: 'mt 10/13/2010 12:55:38.571'!
columnCountFor: aParentModelIndex

	self subclassResponsibility.! !

!UiModel methodsFor: 'signals' stamp: 'mt 10/14/2010 10:20:34.546'!
columnsChangedFor: aParentModelIndex from: start to: end
	"Emitted whenever all model indices in some columns need are invalid."

	self emit.! !

!UiModel methodsFor: 'signals' stamp: 'mt 10/13/2010 13:12:46.124'!
columnsInsertedFor: aParentModelIndex from: start to: end

	self emit.! !

!UiModel methodsFor: 'signals' stamp: 'mt 10/13/2010 13:12:56.304'!
columnsRemovedFor: aParentModelIndex from: start to: end

	self emit.! !

!UiModel methodsFor: 'signals' stamp: 'mt 10/14/2010 10:21:09.141'!
dataChangedFrom: topLeftModelIndex to: bottomRightModelIndex

	self emit.! !

!UiModel methodsFor: 'testing' stamp: 'mt 10/10/2010 10:26:18.507'!
hasChildrenFor: aModelIndex

	^ (self rowCountFor: aModelIndex) > 0! !

!UiModel methodsFor: 'model data' stamp: 'mt 10/13/2010 12:56:42.192'!
iconFor: aModelIndex

	^ nil! !

!UiModel methodsFor: 'index generation' stamp: 'mt 10/13/2010 13:02:11.53'!
indexRow: aRowNumber column: aColumnNumber parent: aParentModelIndex
	"A 'nil' parent indicates the root node. Vertical header indices have 'aColumnNumber = 0'; horizontal header indices have 'aRowNumber = 0'."
	
	self subclassResponsibility.! !

!UiModel methodsFor: 'signals' stamp: 'mt 10/13/2010 12:54:02.23'!
modelReset
	"The internal state has been invalidated. A view needs to refetch all indices."
	
	self emit.! !

!UiModel methodsFor: 'initialization' stamp: 'mt 10/13/2010 12:57:50.04'!
reset

	self modelReset.! !

!UiModel methodsFor: 'counts' stamp: 'mt 10/13/2010 12:55:43.688'!
rowCountFor: aParentModelIndex

	self subclassResponsibility.! !

!UiModel methodsFor: 'signals' stamp: 'mt 10/14/2010 10:20:20.707'!
rowsChangedFor: aParentModelIndex from: start to: end
	"Emitted whenever all model indices in some rows need are invalid."
	
	self emit.! !

!UiModel methodsFor: 'signals' stamp: 'mt 10/13/2010 13:13:06.617'!
rowsInsertedFor: aParentModelIndex from: start to: end

	self emit.! !

!UiModel methodsFor: 'signals' stamp: 'mt 10/13/2010 13:13:02.4'!
rowsRemovedFor: aParentModelIndex from: start to: end

	self emit.! !

!UiModel methodsFor: 'model data' stamp: 'mt 10/13/2010 12:56:45.936'!
textFor: aModelIndex

	^ nil! !

!UiSimpleItemDelegate methodsFor: 'building' stamp: 'mt 10/13/2010 14:42:42.25'!
itemFor: aModelIndex

	| icon text item offset textMorph |
	icon := aModelIndex icon "ifNil: [Morph new extent: 16@16; color: Color transparent]".
	text := (aModelIndex text ifNil: [' ']) asText.
	
	textMorph := text asMorph lock.
	
	item := Morph new
		color: Color transparent;
		height: textMorph height. "for smaller items use font height"
	
	icon ifNotNil: [item addMorphBack: icon asMorph].
	item addMorphBack: textMorph.
	
	"Store morphs in properties for later access."
	item
		setProperty: #text toValue: text;
		setProperty: #icon toValue: icon.
	
	"Manual layouting."
	offset := item left + 3.
	item submorphs do: [:m |
		m left: offset.
		offset := offset + m width + 2.
		m top: (item top - (m height - item height // 2))].
	
	"Expand item horizontally to its contents."
	item
		width: item submorphBounds width + 10; "font rendering bugfix"
		height: item height + 1. "font rendering bugfix"
		
	^ item! !

!UiViewInfo methodsFor: 'accessing' stamp: 'mt 10/12/2010 11:57:56.969'!
headerItem: aMorph

	| backgroundColor |
	backgroundColor := Color gray: 0.85.
	aMorph
		fillStyle: ((GradientFillStyle ramp: {
			0.0 -> Color white.
			0.1-> (backgroundColor adjustBrightness: 0.05).
			0.6 -> (backgroundColor darker)})
			origin: aMorph topLeft;
			direction: 0 @ aMorph height);
		borderStyle: (BorderStyle raised width: 1).
		
	headerItem := aMorph.! !

!UiViewInfo methodsFor: 'accessing' stamp: 'mt 10/13/2010 15:42:39.671'!
itemAt: aModelIndex

	^ items detect: [:item | (item valueOfProperty: #modelIndex) = aModelIndex]! !

!UiTreeViewport methodsFor: 'add/remove' stamp: 'mt 10/13/2010 16:32:59.912'!
addMorph: aMorph atIndex: aModelIndex

	| col row indent |	
	aMorph setProperty: #modelIndex toValue: aModelIndex.
	aMorph setProperty: #isExpanded toValue: false.
	aMorph setProperty: #hasChildren toValue: aModelIndex hasChildren.
	
	row := self addRowFor: aModelIndex.
	row items add: aMorph.
	
	col := self columnFor: aModelIndex.
	col items add: aMorph beforeIndex: (rows indexOf: row).
	
	self addMorphBack: aMorph.
	
	"Correct indent."
	indent := 0.
	(aModelIndex column = 1) ifTrue: [
		(col itemBefore: aMorph)
			ifNil: [indent := self indent] "root"
			ifNotNil: [:item |
				"The item before is my parent. Add additional indentation."
				(item valueOfProperty: #modelIndex) = aModelIndex parent
					ifTrue: [indent := (item valueOfProperty: #indent) + self indent].
				"The item before is my sibling. Use its indentation."
				(item valueOfProperty: #modelIndex) parent = aModelIndex parent
					ifTrue: [indent := item valueOfProperty: #indent]]].
	aMorph setProperty: #indent toValue: indent.! !

!UiTreeViewport methodsFor: 'add/remove - private' stamp: 'mt 10/14/2010 10:02:41.166'!
addRowFor: aModelIndex
	"Adds a new row if needed. Returns the correct one if possible."
	
	| row isCorrectRow isPreviousSibling isParentRow |
	isCorrectRow := [:r | r parentModelIndex = aModelIndex parent
		and: [r firstModelIndex row = aModelIndex row]].
	isPreviousSibling := [:r | r parentModelIndex = aModelIndex parent 
		and: [r firstModelIndex row = (aModelIndex row - 1)]].
	isParentRow := [:r | r firstModelIndex = aModelIndex parent].
	
	rows reverseDetect: [:r |
		(isCorrectRow value: r)
			ifTrue: [row := r. true]
			ifFalse: [((isPreviousSibling value: r) or: [isParentRow value: r])
				ifTrue: [row := rows add: UiViewRowInfo new after: r. true]
				ifFalse: [false]]
		] ifNone: [row := rows add: UiViewRowInfo new].
	
	^ row! !

!UiTreeViewport methodsFor: 'add/remove' stamp: 'mt 10/14/2010 09:45:01.756'!
collapseRow: aRowInfo

	| foundFirst predecessors |
	foundFirst := false.
	predecessors := {aRowInfo} asOrderedCollection.
	rows
		select: [:row |
			"Find all rows and subrows that need to be removed."
			(predecessors anySatisfy: [:pred | row parentModelIndex = pred firstModelIndex])
				ifTrue: [predecessors add: row. foundFirst := true]
				ifFalse: [foundFirst ifTrue: [predecessors := #()]. false]]
		thenDo: [:row | self removeRow: row].! !

!UiTreeViewport methodsFor: 'iterating' stamp: 'mt 10/14/2010 10:25:48.766'!
columnsDo: aBlock

	columns do: aBlock.! !

!UiTreeViewport methodsFor: 'testing' stamp: 'mt 10/14/2010 10:50:19.462'!
isExpanded: aModelIndex

	^ aModelIndex isNil "the root is always expanded"
		or: [(self rowFor: aModelIndex) isExpanded]! !

!UiTreeViewport methodsFor: 'add/remove - private' stamp: 'mt 10/14/2010 09:44:30.676'!
removeRow: aRowInfo

		aRowInfo items withIndexDo: [:item :index |
			(columns at: index) items remove: item].
		
		self removeAllMorphsIn: aRowInfo items.
		rows remove: aRowInfo.! !

!UiTreeViewport methodsFor: 'add/remove' stamp: 'mt 10/14/2010 10:45:00.46'!
replaceMorphWith: aMorph atIndex: aModelIndex

	| oldMorph col row indent |
	oldMorph := (rows detect: [:rowInfo | rowInfo includes: aModelIndex]) itemAt: aModelIndex.
	
	aMorph setProperty: #modelIndex toValue: aModelIndex.
	aMorph setProperty: #isExpanded toValue: (oldMorph valueOfProperty: #isExpanded).
	aMorph setProperty: #hasChildren toValue: aModelIndex hasChildren.
	
	row := self rowFor: aModelIndex.
	row items
		add: aMorph after: oldMorph;
		remove: oldMorph.
	
	col := self columnFor: aModelIndex.
	col items
		add: aMorph after: oldMorph;
		remove: oldMorph.
	
	self
		removeMorph: oldMorph;
		addMorphBack: aMorph.
	
	"Correct indent."
	indent := 0.
	(aModelIndex column = 1) ifTrue: [
		(col itemBefore: aMorph)
			ifNil: [indent := self indent] "root"
			ifNotNil: [:item |
				"The item before is my parent. Add additional indentation."
				(item valueOfProperty: #modelIndex) = aModelIndex parent
					ifTrue: [indent := (item valueOfProperty: #indent) + self indent].
				"The item before is my sibling. Use its indentation."
				(item valueOfProperty: #modelIndex) parent = aModelIndex parent
					ifTrue: [indent := item valueOfProperty: #indent]]].
	aMorph setProperty: #indent toValue: indent.! !

!UiTreeViewport methodsFor: 'iterating' stamp: 'mt 10/14/2010 10:25:55.249'!
rowsDo: aBlock

	rows do: aBlock.! !

!UiTreeViewport methodsFor: 'add/remove' stamp: 'mt 10/12/2010 11:50:38.889'!
setHeaderItem: aMorph atRow: anInteger

	(rows at: anInteger) headerItem: aMorph.! !

!UiModelIndex methodsFor: 'comparing' stamp: 'mt 10/13/2010 15:07:28.4'!
hash

	^ super hash bitXor: self parent hash! !

!UiSmalltalkModel methodsFor: 'counts' stamp: 'mt 10/11/2010 16:30:56.086'!
columnCountFor: aParentModelIndex
	"name, childCount, type"
	
	^ 3! !

!UiSmalltalkModel methodsFor: 'model data' stamp: 'mt 10/11/2010 16:34:34.161'!
iconFor: aModelIndex

	| col package systemCategory class protocol method |
	col := aModelIndex column.
	package := aModelIndex valueOfProperty: #package ifAbsent: [].
	systemCategory := aModelIndex valueOfProperty: #systemCategory  ifAbsent: [].
	class := aModelIndex valueOfProperty: #class ifAbsent: [].
	protocol := aModelIndex valueOfProperty: #protocol ifAbsent: [].
	method := aModelIndex valueOfProperty: #method ifAbsent: [].
	
	"Horizontal headers."
	aModelIndex row = 0 ifTrue: [
		col = 1 ifTrue: [^ UiSilkIcons fontIcon].
		col = 2 ifTrue: [^ UiSilkIcons chartBarIcon].
		col = 3 ifTrue: [^ UiSilkIcons informationIcon]].
	
	package ifNotNil: [:p | 
		col = 1 ifTrue: [^ UiSilkIcons packageIcon]].
	
	"systemCategory ifNotNil: [:cat | ].
	
	class ifNotNil: [:c |].

	protocol ifNotNil: [:p | ].
	
	method ifNotNil: [:m | ]."
		
	^ nil! !

!UiSmalltalkModel methodsFor: 'index generation' stamp: 'mt 10/10/2010 14:19:12.733'!
indexRow: aRowNumber column: aColumnNumber parent: aParentModelIndex

	| index |
	index := (UiSimpleModelIndex row: aRowNumber column: aColumnNumber)
		model: self;
		parent: aParentModelIndex.
	
	"Header indices."
	(aRowNumber = 0 or: [aColumnNumber = 0])
		ifTrue: [^ index].
	
	aParentModelIndex ifNil: [
		index setProperty: #package toValue: (PackageOrganizer default packages at: aRowNumber).
		^ index].
	
	(aParentModelIndex includesProperty: #package) ifTrue: [
		index
			setProperty: #systemCategory
			toValue: ((aParentModelIndex valueOfProperty: #package) systemCategories at: aRowNumber).
		index
			setProperty: #systemCategoryPrefix
			toValue: (aParentModelIndex valueOfProperty: #package) systemCategoryPrefix.
		^ index].
	
	(aParentModelIndex includesProperty: #systemCategory) ifTrue: [
		index
			setProperty: #class
			toValue: ((SystemOrganization
				listAtCategoryNamed: (aParentModelIndex valueOfProperty: #systemCategory))
				at: aRowNumber).
		^ index].
	
	(aParentModelIndex includesProperty: #class) ifTrue: [ | class |
		class := Smalltalk at: (aParentModelIndex valueOfProperty: #class).
		index
			setProperty: #organization
			toValue: class organization.
		index
			setProperty: #protocol
			toValue: (class organization categories at: aRowNumber).
		^ index].

	(aParentModelIndex includesProperty: #protocol) ifTrue: [ | organization protocol |
		organization := aParentModelIndex valueOfProperty: #organization.
		protocol := aParentModelIndex valueOfProperty: #protocol.
		index
			setProperty: #method
			toValue: ((organization listAtCategoryNamed: protocol) at: aRowNumber).
		^ index].! !

!UiSmalltalkModel methodsFor: 'initialization' stamp: 'mt 10/13/2010 13:29:47.625'!
initialize

	SystemChangeNotifier uniqueInstance
		notify: self ofAllSystemChangesUsing: #processNotification:.! !

!UiSmalltalkModel methodsFor: 'notifications' stamp: 'mt 10/14/2010 11:19:44.271'!
processNotification: anEvent

	| pkg pkgIndex sysIndex classIndex |
	(anEvent itemKind ~= #category and: [anEvent itemKind ~= #class])
		ifTrue: [^ self].
	
	pkg := PackageOrganizer default
		packageOfSystemCategory: anEvent itemCategory
		ifNone: [^ self].
	pkgIndex := (UiSimpleModelIndex
		row: (PackageOrganizer default packages indexOf: pkg)
		column: 1)
		model: self;
		setProperty: #package toValue: pkg.
	sysIndex := self
		indexRow: (pkg systemCategories indexOf: anEvent itemCategory)
		column: 1
		parent: pkgIndex.
	classIndex := self
		indexRow: ((SystemOrganization listAtCategoryNamed: anEvent itemCategory) indexOf: anEvent itemClass name)
		column: 1
		parent: sysIndex.
	
	anEvent itemKind = #category ifTrue: [
		anEvent isAdded ifTrue: [
			self rowsInsertedFor: pkgIndex from: sysIndex row to: sysIndex row.
			self rowsChangedFor: nil from: pkgIndex row to: pkgIndex row].
		anEvent isRemoved ifTrue: [self rowsRemovedFor: pkgIndex from: 1 to: 1].
		anEvent isRenamed ifTrue: [
			self rowsChangedFor: nil from: pkgIndex row to: pkgIndex row]].

	anEvent itemKind = #class ifTrue: [
		anEvent isAdded ifTrue: [
			self rowsInsertedFor: sysIndex from: classIndex row to: classIndex row.
			self rowsChangedFor: pkgIndex from: sysIndex row to: sysIndex row]
		].
	
	anEvent itemKind = #method ifTrue: [
		].! !

!UiSmalltalkModel methodsFor: 'counts' stamp: 'mt 10/10/2010 14:13:30.029'!
rowCountFor: aParentModelIndex
	
	aParentModelIndex ifNil: [
		^ PackageOrganizer default packages size].
	
	aParentModelIndex column ~= 1 ifTrue: [^ 0].
	
	(aParentModelIndex includesProperty: #package) ifTrue: [
		^ (aParentModelIndex valueOfProperty: #package) systemCategories size].
	
	(aParentModelIndex includesProperty: #systemCategory) ifTrue: [
		^ (SystemOrganization
				listAtCategoryNamed: (aParentModelIndex valueOfProperty: #systemCategory)) size].
	
	(aParentModelIndex includesProperty: #class) ifTrue: [ | class |
		^ (Smalltalk at: (aParentModelIndex valueOfProperty: #class)) organization categories size].

	(aParentModelIndex includesProperty: #protocol) ifTrue: [ | organization protocol |
		organization := aParentModelIndex valueOfProperty: #organization.
		protocol := aParentModelIndex valueOfProperty: #protocol.
		^ (organization listAtCategoryNamed: protocol) size].
	
	^ 0! !

!UiSmalltalkModel methodsFor: 'model data' stamp: 'mt 10/11/2010 16:35:40.297'!
textFor: aModelIndex

	| col package systemCategory class protocol method |
	col := aModelIndex column.
	package := aModelIndex valueOfProperty: #package ifAbsent: [].
	systemCategory := aModelIndex valueOfProperty: #systemCategory  ifAbsent: [].
	class := aModelIndex valueOfProperty: #class ifAbsent: [].
	protocol := aModelIndex valueOfProperty: #protocol ifAbsent: [].
	method := aModelIndex valueOfProperty: #method ifAbsent: [].
	
	"Horizontal headers."
	aModelIndex row = 0 ifTrue: [
		col = 1 ifTrue: [^ '<b>Name</b>' asHtmlText].
		col = 2 ifTrue: [^ '<b>Size</b>' asHtmlText].
		col = 3 ifTrue: [^ '<b>Type</b>' asHtmlText]].
	
	package ifNotNil: [:p | 
		col = 1 ifTrue: [^ p name].
		col = 2 ifTrue: [^ ('<font color="#555555">', p systemCategories size asString, '</font>') asHtmlText].
		col = 3 ifTrue: [^ '<font color="#555555">package</font>' asHtmlText]].
	
	systemCategory ifNotNil: [:cat | 
		| prefix |
		prefix := aModelIndex valueOfProperty: #systemCategoryPrefix.
		
		col = 2 ifTrue: [^ ('<font color="#555555">', (SystemOrganization listAtCategoryNamed: cat) size asString, '</font>') asHtmlText].
		
		cat = prefix ifFalse: [cat := cat allButFirst: prefix size + 1].
		col = 1 ifTrue: [^ cat].
		col = 3 ifTrue: [^ '<font color="#555555">system category</font>' asHtmlText]].
	
	class ifNotNil: [:c |
		col = 1 ifTrue: [^ c].
		col = 2 ifTrue: [^ ('<font color="#555555">', (Smalltalk at: c) organization categories size asString, '</font>') asHtmlText].
		col = 3 ifTrue: [^ '<font color="#555555">class</font>' asHtmlText]].

	protocol ifNotNil: [:p |
		| organization |
		organization := aModelIndex valueOfProperty: #organization.
		col = 1 ifTrue: [^ p].
		col = 2 ifTrue: [^ ('<font color="#555555">', (organization listAtCategoryNamed: p) size asString, '</font>') asHtmlText].
		col = 3 ifTrue: [^ '<font color="#555555">protocol</font>' asHtmlText]].
	
	method ifNotNil: [:m |
		col = 1 ifTrue: [^ '#', m].
		col = 3 ifTrue: [^ '<font color="#555555">method</font>' asHtmlText]].
		
	^ nil! !
