
!UiTreeView methodsFor: 'as yet unclassified' stamp: 'mt 10/11/2010 18:05:14.956'!
fillViewportFor: aModelIndex animated: aBoolean

	| index item |
	self model ifNil: [^ self].
	
	1 to: (self model rowCountFor: aModelIndex) do: [:row |
		1 to: (self model columnCountFor: aModelIndex) do: [:col |
			index := self model indexRow: row column: col parent: aModelIndex.
			item := (self itemDelegateFor: index) itemFor: index.
			self viewport addMorph: item atIndex: index.
			aBoolean ifTrue: [item fadeIn]]].! !

!UiTreeView methodsFor: 'initialization' stamp: 'mt 10/10/2010 10:42:28.991'!
model: aModel

	super model: aModel.
	self reset.! !

!UiTreeView methodsFor: 'as yet unclassified' stamp: 'mt 10/10/2010 14:15:33.809'!
reset
	self viewport clear.
	
	self fillViewportFor: nil animated: false.
	self resetHeaderItems.
	
	self viewport
		updateLayout;
		updateExtent.
		
	self updateScrollBars.! !

!UiTreeView methodsFor: 'as yet unclassified' stamp: 'mt 10/11/2010 18:05:08.738'!
resetHeaderItems

	| index item |
	self model ifNil: [^ self].
	"1 to: (self model rowCountFor: nil) do: [:row |
		index := self model indexRow: row column: 0 parent: nil.
		item := (self itemDelegateFor: index) itemFor: index.
		item width: verticalHeader width.
		self viewport setHeaderItem: item atRow: row]."
	
	1 to: (self model columnCountFor: nil) do: [:col |
		index := self model indexRow: 0 column: col parent: nil.
		item := (self itemDelegateFor: index) itemFor: index.
		horizontalHeader height: (horizontalHeader height max: item height).
		self viewport setHeaderItem: item atColumn: col].! !

!UiTreeView methodsFor: 'as yet unclassified' stamp: 'mt 10/10/2010 12:04:23.588'!
toggleIndex: aModelIndex

	| row |
	
	row := self viewport rowFor: aModelIndex.
	
	row isExpanded
		ifTrue: [
			row items first setProperty: #isExpanded toValue: false.
			self viewport collapseRow: row]
		ifFalse: [
			row items first setProperty: #isExpanded toValue: true.
			self
				fillViewportFor: row firstModelIndex
				animated: true].
	
	self viewport
		updateLayout;
		updateExtent.
		
	self updateScrollBars.! !

!UiModel methodsFor: 'accessing' stamp: 'mt 10/7/2010 18:02:41.543'!
columnCountFor: aParentModelIndex! !

!UiModel methodsFor: 'accessing' stamp: 'mt 10/10/2010 10:26:18.507'!
hasChildrenFor: aModelIndex

	^ (self rowCountFor: aModelIndex) > 0! !

!UiModel methodsFor: 'accessing - data' stamp: 'mt 10/7/2010 18:01:10.433'!
iconFor: aModelIndex! !

!UiModel methodsFor: 'accessing' stamp: 'mt 10/7/2010 18:08:49.323'!
indexRow: aRowNumber column: aColumnNumber parent: aParentModelIndex! !

!UiModel methodsFor: 'accessing' stamp: 'mt 10/7/2010 18:03:23.332'!
rowCountFor: aParentModelIndex! !

!UiModel methodsFor: 'accessing - data' stamp: 'mt 10/7/2010 18:01:05.609'!
textFor: aModelIndex! !

!UiSimpleItemDelegate methodsFor: 'building' stamp: 'mt 10/11/2010 16:30:49.945'!
itemFor: aModelIndex

	| icon text item offset |
	icon := aModelIndex icon "ifNil: [Morph new extent: 16@16; color: Color transparent]".
	text := (aModelIndex text
		ifNil: [' ' asText asMorph lock]
		ifNotNil: [:t | t asText asMorph lock]).
	item := Morph new
		color: Color transparent;
		height: text height. "for smaller items use font height"
	
	icon ifNotNil: [item addMorphBack: icon asMorph].
	item addMorphBack: text.
	
	"Manual layouting."
	offset := item left + 3.
	item submorphs do: [:m |
		m left: offset.
		offset := offset + m width + 2.
		m top: (item top - (m height - item height // 2))].
	
	"Expand item horizontally to its contents."
	item
		width: item submorphBounds width + 10; "font rendering bugfix"
		height: item height + 1. "font rendering bugfix"
		
	^ item! !

!UiViewInfo methodsFor: 'accessing' stamp: 'mt 10/10/2010 14:07:16.803'!
headerItem: aMorph

	aMorph
		color: (Color gray: 0.85);
		borderStyle: (BorderStyle raised width: 1).
		
	headerItem := aMorph.! !

!UiTreeViewport methodsFor: 'add/remove' stamp: 'mt 10/10/2010 11:35:20.713'!
addMorph: aMorph atIndex: aModelIndex

	| col row indent |	
	aMorph setProperty: #modelIndex toValue: aModelIndex.
	aMorph setProperty: #isExpanded toValue: false.
	aMorph setProperty: #hasChildren toValue: aModelIndex hasChildren.
	
	row := self addRowFor: aModelIndex.
	row items add: aMorph.
	
	col := self columnFor: aModelIndex.
	col items add: aMorph beforeIndex: (rows indexOf: row).
	
	self addMorphBack: aMorph.
	
	"Correct indent."
	indent := 0.
	(aModelIndex column = 1) ifTrue: [
		(col itemBefore: aMorph)
			ifNil: [indent := self indent] "root"
			ifNotNil: [:item |
				"The item before is my parent. Add additional indentation."
				(item valueOfProperty: #modelIndex) = aModelIndex parent
					ifTrue: [indent := (item valueOfProperty: #indent) + self indent].
				"The item before is my sibling. Use its indentation."
				(item valueOfProperty: #modelIndex) parent = aModelIndex parent
					ifTrue: [indent := item valueOfProperty: #indent]]].
	aMorph setProperty: #indent toValue: indent.! !

!UiTreeViewport methodsFor: 'add/remove' stamp: 'mt 10/10/2010 14:34:26.522'!
addRowFor: aModelIndex

	| row |
	aModelIndex column = 1
		ifTrue: ["Create new row."
			rows
				ifEmpty: [row := rows add: UiViewRowInfo new]
				ifNotEmpty: [row := rows add: UiViewRowInfo new after: (rows
					reverseDetect: [:r |
						"sibling row"
						r parentModelIndex = aModelIndex parent
						"parent row"
						or: [r firstModelIndex = aModelIndex parent]])]]
		ifFalse: ["Find the row."
			row := rows
				reverseDetect: [:r | r parentModelIndex = aModelIndex parent]].
	
	^ row! !

!UiTreeViewport methodsFor: 'add/remove' stamp: 'mt 10/10/2010 10:54:27.193'!
collapseRow: aRowInfo

	| foundFirst predecessors |
	foundFirst := false.
	predecessors := {aRowInfo} asOrderedCollection.
	rows
		select: [:row |
			(predecessors anySatisfy: [:pred | row parentModelIndex = pred firstModelIndex])
				ifTrue: [predecessors add: row. foundFirst := true]
				ifFalse: [foundFirst ifTrue: [predecessors := #()]. false]]
		thenDo: [:row |
			row items withIndexDo: [:item :index | (columns at: index) items remove: item].
			self removeAllMorphsIn: row items.
			rows remove: row].! !

!UiTreeViewport methodsFor: 'add/remove' stamp: 'mt 10/10/2010 13:58:39.037'!
setHeaderItem: aMorph atRow: anInteger

	"No custom row labels yet."! !

!UiSmalltalkModel methodsFor: 'accessing' stamp: 'mt 10/11/2010 16:30:56.086'!
columnCountFor: aParentModelIndex
	"name, childCount, type"
	
	^ 3! !

!UiSmalltalkModel methodsFor: 'accessing - data' stamp: 'mt 10/11/2010 16:34:34.161'!
iconFor: aModelIndex

	| col package systemCategory class protocol method |
	col := aModelIndex column.
	package := aModelIndex valueOfProperty: #package ifAbsent: [].
	systemCategory := aModelIndex valueOfProperty: #systemCategory  ifAbsent: [].
	class := aModelIndex valueOfProperty: #class ifAbsent: [].
	protocol := aModelIndex valueOfProperty: #protocol ifAbsent: [].
	method := aModelIndex valueOfProperty: #method ifAbsent: [].
	
	"Horizontal headers."
	aModelIndex row = 0 ifTrue: [
		col = 1 ifTrue: [^ UiSilkIcons fontIcon].
		col = 2 ifTrue: [^ UiSilkIcons chartBarIcon].
		col = 3 ifTrue: [^ UiSilkIcons informationIcon]].
	
	package ifNotNil: [:p | 
		col = 1 ifTrue: [^ UiSilkIcons packageIcon]].
	
	"systemCategory ifNotNil: [:cat | ].
	
	class ifNotNil: [:c |].

	protocol ifNotNil: [:p | ].
	
	method ifNotNil: [:m | ]."
		
	^ nil! !

!UiSmalltalkModel methodsFor: 'accessing' stamp: 'mt 10/10/2010 14:19:12.733'!
indexRow: aRowNumber column: aColumnNumber parent: aParentModelIndex

	| index |
	index := (UiSimpleModelIndex row: aRowNumber column: aColumnNumber)
		model: self;
		parent: aParentModelIndex.
	
	"Header indices."
	(aRowNumber = 0 or: [aColumnNumber = 0])
		ifTrue: [^ index].
	
	aParentModelIndex ifNil: [
		index setProperty: #package toValue: (PackageOrganizer default packages at: aRowNumber).
		^ index].
	
	(aParentModelIndex includesProperty: #package) ifTrue: [
		index
			setProperty: #systemCategory
			toValue: ((aParentModelIndex valueOfProperty: #package) systemCategories at: aRowNumber).
		index
			setProperty: #systemCategoryPrefix
			toValue: (aParentModelIndex valueOfProperty: #package) systemCategoryPrefix.
		^ index].
	
	(aParentModelIndex includesProperty: #systemCategory) ifTrue: [
		index
			setProperty: #class
			toValue: ((SystemOrganization
				listAtCategoryNamed: (aParentModelIndex valueOfProperty: #systemCategory))
				at: aRowNumber).
		^ index].
	
	(aParentModelIndex includesProperty: #class) ifTrue: [ | class |
		class := Smalltalk at: (aParentModelIndex valueOfProperty: #class).
		index
			setProperty: #organization
			toValue: class organization.
		index
			setProperty: #protocol
			toValue: (class organization categories at: aRowNumber).
		^ index].

	(aParentModelIndex includesProperty: #protocol) ifTrue: [ | organization protocol |
		organization := aParentModelIndex valueOfProperty: #organization.
		protocol := aParentModelIndex valueOfProperty: #protocol.
		index
			setProperty: #method
			toValue: ((organization listAtCategoryNamed: protocol) at: aRowNumber).
		^ index].! !

!UiSmalltalkModel methodsFor: 'accessing' stamp: 'mt 10/10/2010 14:13:30.029'!
rowCountFor: aParentModelIndex
	
	aParentModelIndex ifNil: [
		^ PackageOrganizer default packages size].
	
	aParentModelIndex column ~= 1 ifTrue: [^ 0].
	
	(aParentModelIndex includesProperty: #package) ifTrue: [
		^ (aParentModelIndex valueOfProperty: #package) systemCategories size].
	
	(aParentModelIndex includesProperty: #systemCategory) ifTrue: [
		^ (SystemOrganization
				listAtCategoryNamed: (aParentModelIndex valueOfProperty: #systemCategory)) size].
	
	(aParentModelIndex includesProperty: #class) ifTrue: [ | class |
		^ (Smalltalk at: (aParentModelIndex valueOfProperty: #class)) organization categories size].

	(aParentModelIndex includesProperty: #protocol) ifTrue: [ | organization protocol |
		organization := aParentModelIndex valueOfProperty: #organization.
		protocol := aParentModelIndex valueOfProperty: #protocol.
		^ (organization listAtCategoryNamed: protocol) size].
	
	^ 0! !

!UiSmalltalkModel methodsFor: 'accessing - data' stamp: 'mt 10/11/2010 16:35:40.297'!
textFor: aModelIndex

	| col package systemCategory class protocol method |
	col := aModelIndex column.
	package := aModelIndex valueOfProperty: #package ifAbsent: [].
	systemCategory := aModelIndex valueOfProperty: #systemCategory  ifAbsent: [].
	class := aModelIndex valueOfProperty: #class ifAbsent: [].
	protocol := aModelIndex valueOfProperty: #protocol ifAbsent: [].
	method := aModelIndex valueOfProperty: #method ifAbsent: [].
	
	"Horizontal headers."
	aModelIndex row = 0 ifTrue: [
		col = 1 ifTrue: [^ '<b>Name</b>' asHtmlText].
		col = 2 ifTrue: [^ '<b>Size</b>' asHtmlText].
		col = 3 ifTrue: [^ '<b>Type</b>' asHtmlText]].
	
	package ifNotNil: [:p | 
		col = 1 ifTrue: [^ p name].
		col = 2 ifTrue: [^ ('<font color="#555555">', p systemCategories size asString, '</font>') asHtmlText].
		col = 3 ifTrue: [^ '<font color="#555555">package</font>' asHtmlText]].
	
	systemCategory ifNotNil: [:cat | 
		| prefix |
		prefix := aModelIndex valueOfProperty: #systemCategoryPrefix.
		
		col = 2 ifTrue: [^ ('<font color="#555555">', (SystemOrganization listAtCategoryNamed: cat) size asString, '</font>') asHtmlText].
		
		cat = prefix ifFalse: [cat := cat allButFirst: prefix size + 1].
		col = 1 ifTrue: [^ cat].
		col = 3 ifTrue: [^ '<font color="#555555">system category</font>' asHtmlText]].
	
	class ifNotNil: [:c |
		col = 1 ifTrue: [^ c].
		col = 2 ifTrue: [^ ('<font color="#555555">', (Smalltalk at: c) organization categories size asString, '</font>') asHtmlText].
		col = 3 ifTrue: [^ '<font color="#555555">class</font>' asHtmlText]].

	protocol ifNotNil: [:p |
		| organization |
		organization := aModelIndex valueOfProperty: #organization.
		col = 1 ifTrue: [^ p].
		col = 2 ifTrue: [^ ('<font color="#555555">', (organization listAtCategoryNamed: p) size asString, '</font>') asHtmlText].
		col = 3 ifTrue: [^ '<font color="#555555">protocol</font>' asHtmlText]].
	
	method ifNotNil: [:m |
		col = 1 ifTrue: [^ '#', m].
		col = 3 ifTrue: [^ '<font color="#555555">method</font>' asHtmlText]].
		
	^ nil! !
